## View library for Javascript



# Views


## Base View

The default concrete view class.

### view.setup(options)

Sets the view up with configuration options

### view.instructions()

Initialization instructions used to load the view. Needed for loading 

### view.render()

Called to render the view - display it to the screen.

### view.listen()

Called after a view has been rendered.

### final view.update(force)

Call this if anything changes in the view, such as view data. This method calls: `render`, `listen`, then `updateChildren`.

### view.$

View level accessor.

### view.addChild(child)

Adds a child view

### view.removeAllChildren()

Removes all chidren from the view 


### Example

```javascript

var views = require('fig').views;


var FigView = views.View({
   
   /**
    * event bindings 
    */
    
   'bindings': {
       'click #some-button': 'onButtonClick'
   },


   /**
    * Sets up the view - called from the constructor
    */


   'override setup': function(options) {
       this._super(options);

       //called on constructor
   },

   /**
    */

    'override listen': function(options) {
        this._super(options);

        //listen to stuff
    },

    /**
     */

    'override instructions': function() {
        return this._super().concat('_loadSomeInitData');
    },
   
   /**
    */


   '_loadSomeInitData': function(next) {
        setTimeout(next, 1000);
    },


   /**
    */

   'onButtonClick': function() {
       //do stuff
   }
});

```


## Item View

Item view which hooks up to a remote item viw beanpoll

## Collection View

Collection view which hooks up to a remote collection.

## Template View

Template view which expects a template file, or template element.

## ItemTemplate

Template + Item view

## CollectionTemplate

Template + Collection view

## SelectView

Select Option view (TODO)

